"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
        return extendStatics(d, b);
    };
    return function (d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
Object.defineProperty(exports, "__esModule", { value: true });
var OAuth2Strategy = require("passport-oauth2");
// Create a new class that extends OAuth2Strategy
var InternalOAuthError = OAuth2Strategy.InternalOAuthError;
var YahooOauthTokenStrategy = /** @class */ (function (_super) {
    __extends(YahooOauthTokenStrategy, _super);
    function YahooOauthTokenStrategy(options, verify) {
        var _this = this;
        var _a, _b, _c;
        // Append default options
        var preparedOptions = __assign(__assign({}, options), { tokenURL: (_a = options.tokenURL) !== null && _a !== void 0 ? _a : "https://api.login.yahoo.com/oauth2/get_token", authorizationURL: (_b = options.authorizationURL) !== null && _b !== void 0 ? _b : "https://api.login.yahoo.com/oauth2/request_auth" });
        // Call parent constructor
        _this = _super.call(this, preparedOptions, verify) || this;
        // Set name
        _this.name = 'yahoo-oauth-token';
        // Set profile URL
        _this._profileURL = options.profileURL || "https://api.login.yahoo.com/openid/v1/userinfo";
        // Set access token field and passReqToCallback
        _this._accessTokenField = options.accessTokenField || 'access_token';
        _this._accessCodeField = options.accessCodeField || 'code';
        _this._refreshTokenField = options.refreshTokenField || 'refresh_token';
        _this._passReqToCallback = (_c = options.passReqToCallback) !== null && _c !== void 0 ? _c : false;
        // Set OAuth2 options
        _this._oauth2.useAuthorizationHeaderforGET(true);
        return _this;
    }
    /**
     * Authenticate request by delegating to a service provider using OAuth 2.0.
     * @param {Object} req
     * @param {Object} _options
     */
    YahooOauthTokenStrategy.prototype.authenticate = function (req, _options) {
        var _this = this;
        var accessCode = this.lookup(req, this._accessCodeField);
        var accessToken = this.lookup(req, this._accessTokenField);
        var refreshToken = this.lookup(req, this._refreshTokenField);
        if (!accessCode && !accessToken)
            return this.fail({ message: "You should provide ".concat(this._accessCodeField, " or ").concat(this._accessTokenField) });
        if (!accessToken) {
            // If no access token is present, try to obtain an access token using an authorization code.
            this._oauth2.getOAuthAccessToken(accessCode, {
                grant_type: 'authorization_code',
                redirect_uri: 'oob', // Ignore redirect uri
            }, function (error, accessTokenExchanged, refreshTokenExchanged, _results) {
                if (error)
                    return _this.error(error);
                if (!accessTokenExchanged)
                    return _this.fail({ message: "Error exchanging authorization code for access token" });
                // Call user profile
                _this._returnUserProfile(accessTokenExchanged, refreshTokenExchanged !== null && refreshTokenExchanged !== void 0 ? refreshTokenExchanged : refreshToken, req);
            });
        }
        else {
            // Call user profile
            this._returnUserProfile(accessToken, refreshToken, req);
        }
    };
    YahooOauthTokenStrategy.prototype._returnUserProfile = function (accessToken, refreshToken, req) {
        var _this = this;
        // Load user profile
        this._loadUserProfile(accessToken, function (errorUserProfile, profile) {
            if (errorUserProfile)
                return _this.error(errorUserProfile);
            var verified = function (error, user, info) {
                if (error)
                    return _this.error(error);
                if (!user)
                    return _this.fail(info);
                return _this.success(user, info);
            };
            if (_this._passReqToCallback) {
                _this._verify(req, accessToken, refreshToken, profile, verified);
            }
            else {
                _this._verify(accessToken, refreshToken, profile, verified);
            }
        });
    };
    /**
     * Retrieve user profile from Yahoo.
     *
     * This function constructs a normalized profile, with the following properties:
     *
     *   - `provider`         always set to `yahoo`
     *   - `id`               the user's Yahoo ID
     *   - `username`         the user's Yahoo username
     *   - `name`      the user's full name
     *
     * @param {String} accessToken
     * @param {Function} done
     */
    YahooOauthTokenStrategy.prototype.userProfile = function (accessToken, done) {
        this._oauth2.get(this._profileURL, accessToken, function (error, body, _res) {
            if (error) {
                return done(new InternalOAuthError('Failed to fetch user profile', error));
            }
            try {
                var json = typeof body === 'string' ? JSON.parse(body) : body;
                var profile = YahooOauthTokenStrategy.parseProfile(json);
                profile._raw = body;
                done(null, profile);
            }
            catch (e) {
                done(e);
            }
        });
    };
    /**
     * This method handles searhing the value of provided field in body, query, and header.
     *
     * @param {Object} req http request object
     * @param {String} field
     * @returns {String} field's value in body, query, or headers
     */
    YahooOauthTokenStrategy.prototype.lookup = function (req, field) {
        return (req.body && req.body[field]) || (req.query && req.query[field]) || (req.headers && req.headers[field]);
    };
    /**
     * Parse profile.
     *
     * Parses user profiles as fetched from Yahoo's OpenID Connect-compatible user
     * info endpoint.
     *
     * The amount of detail in the profile varies based on the scopes granted by the
     * user. The following scope values add additional data:
     *
     *     `profile` - basic profile information
     *     `email` - email address
     *
     * References:
     *   - https://developer.yahoo.com/sign-in-with-yahoo/#node-step-four
     *
     * @param {object} json
     * @return {TParsedProfile}
     */
    YahooOauthTokenStrategy.parseProfile = function (json) {
        var profile = __assign(__assign({}, json), { provider: 'yahoo', id: json.sub, email: json.email || '', _json: json });
        if (json.profile_images) {
            profile.photos = Object.entries(json.profile_images).map(function (_a) {
                var type = _a[0], value = _a[1];
                return ({
                    type: type,
                    value: value,
                });
            });
        }
        return profile;
    };
    return YahooOauthTokenStrategy;
}(OAuth2Strategy));
exports.default = YahooOauthTokenStrategy;
